# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Any, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.firewall.waf.packages import group_edit_params, group_list_params
from .....types.firewall.waf.packages.group import Group
from .....types.firewall.waf.packages.group_get_response import GroupGetResponse
from .....types.firewall.waf.packages.group_edit_response import GroupEditResponse

__all__ = ["GroupsResource", "AsyncGroupsResource"]


class GroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return GroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return GroupsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        package_id: str,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        mode: Literal["on", "off"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["mode", "rules_count"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        rules_count: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Group]:
        """
        Fetches the WAF rule groups in a WAF package.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          direction: Defines the direction used to sort returned rule groups.

          match: Defines the condition for search requirements. When set to `all`, all the search
              requirements must match. When set to `any`, only one of the search requirements
              has to match.

          mode: Defines the state of the rules contained in the rule group. When `on`, the rules
              in the group are configurable/usable.

          name: Defines the name of the rule group.

          order: Defines the field used to sort returned rule groups.

          page: Defines the page number of paginated results.

          per_page: Defines the number of rule groups per page.

          rules_count: Defines the number of rules in the current rule group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        return self._get_api_list(
            path_template(
                "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups", zone_id=zone_id, package_id=package_id
            ),
            page=SyncV4PagePaginationArray[Group],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "mode": mode,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "rules_count": rules_count,
                    },
                    group_list_params.GroupListParams,
                ),
            ),
            model=Group,
        )

    @typing_extensions.deprecated("deprecated")
    def edit(
        self,
        group_id: str,
        *,
        zone_id: str,
        package_id: str,
        mode: Literal["on", "off"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupEditResponse:
        """Updates a WAF rule group.

        You can update the state (`mode` parameter) of a rule
        group.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          group_id: Defines the unique identifier of a WAF package.

          mode: Defines the state of the rules contained in the rule group. When `on`, the rules
              in the group are configurable/usable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return cast(
            GroupEditResponse,
            self._patch(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups/{group_id}",
                    zone_id=zone_id,
                    package_id=package_id,
                    group_id=group_id,
                ),
                body=maybe_transform({"mode": mode}, group_edit_params.GroupEditParams),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[GroupEditResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[GroupEditResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        group_id: str,
        *,
        zone_id: str,
        package_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupGetResponse:
        """
        Fetches the details of a WAF rule group.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          group_id: Defines the unique identifier of a WAF package.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return cast(
            GroupGetResponse,
            self._get(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups/{group_id}",
                    zone_id=zone_id,
                    package_id=package_id,
                    group_id=group_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[GroupGetResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[GroupGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncGroupsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        package_id: str,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        mode: Literal["on", "off"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["mode", "rules_count"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        rules_count: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Group, AsyncV4PagePaginationArray[Group]]:
        """
        Fetches the WAF rule groups in a WAF package.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          direction: Defines the direction used to sort returned rule groups.

          match: Defines the condition for search requirements. When set to `all`, all the search
              requirements must match. When set to `any`, only one of the search requirements
              has to match.

          mode: Defines the state of the rules contained in the rule group. When `on`, the rules
              in the group are configurable/usable.

          name: Defines the name of the rule group.

          order: Defines the field used to sort returned rule groups.

          page: Defines the page number of paginated results.

          per_page: Defines the number of rule groups per page.

          rules_count: Defines the number of rules in the current rule group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        return self._get_api_list(
            path_template(
                "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups", zone_id=zone_id, package_id=package_id
            ),
            page=AsyncV4PagePaginationArray[Group],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "mode": mode,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "rules_count": rules_count,
                    },
                    group_list_params.GroupListParams,
                ),
            ),
            model=Group,
        )

    @typing_extensions.deprecated("deprecated")
    async def edit(
        self,
        group_id: str,
        *,
        zone_id: str,
        package_id: str,
        mode: Literal["on", "off"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupEditResponse:
        """Updates a WAF rule group.

        You can update the state (`mode` parameter) of a rule
        group.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          group_id: Defines the unique identifier of a WAF package.

          mode: Defines the state of the rules contained in the rule group. When `on`, the rules
              in the group are configurable/usable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return cast(
            GroupEditResponse,
            await self._patch(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups/{group_id}",
                    zone_id=zone_id,
                    package_id=package_id,
                    group_id=group_id,
                ),
                body=await async_maybe_transform({"mode": mode}, group_edit_params.GroupEditParams),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[GroupEditResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[GroupEditResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        group_id: str,
        *,
        zone_id: str,
        package_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupGetResponse:
        """
        Fetches the details of a WAF rule group.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier of a schema.

          package_id: Defines the unique identifier of a WAF package.

          group_id: Defines the unique identifier of a WAF package.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not package_id:
            raise ValueError(f"Expected a non-empty value for `package_id` but received {package_id!r}")
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return cast(
            GroupGetResponse,
            await self._get(
                path_template(
                    "/zones/{zone_id}/firewall/waf/packages/{package_id}/groups/{group_id}",
                    zone_id=zone_id,
                    package_id=package_id,
                    group_id=group_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[GroupGetResponse]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[GroupGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class GroupsResourceWithRawResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                groups.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                groups.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                groups.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncGroupsResourceWithRawResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                groups.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                groups.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                groups.get,  # pyright: ignore[reportDeprecated],
            )
        )


class GroupsResourceWithStreamingResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                groups.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                groups.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                groups.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncGroupsResourceWithStreamingResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                groups.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                groups.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                groups.get,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["CNIUpdateParams", "Magic", "BGP"]


class CNIUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Customer account tag"""

    id: Required[str]

    account: Required[str]
    """Customer account tag"""

    cust_ip: Required[str]
    """Customer end of the point-to-point link

    This should always be inside the same prefix as `p2p_ip`.
    """

    interconnect: Required[str]
    """Interconnect identifier hosting this CNI"""

    magic: Required[Magic]

    p2p_ip: Required[str]
    """Cloudflare end of the point-to-point link"""

    bgp: BGP


class Magic(TypedDict, total=False):
    conduit_name: Required[str]

    description: Required[str]

    mtu: Required[int]


class BGP(TypedDict, total=False):
    customer_asn: Required[int]
    """ASN used on the customer end of the BGP session"""

    extra_prefixes: Required[SequenceNotStr[str]]
    """Extra set of static prefixes to advertise to the customer's end of the session"""

    md5_key: Optional[str]
    """MD5 key to use for session authentication.

    Note that _this is not a security measure_. MD5 is not a valid security
    mechanism, and the key is not treated as a secret value. This is _only_
    supported for preventing misconfiguration, not for defending against malicious
    attacks.

    The MD5 key, if set, must be of non-zero length and consist only of the
    following types of character:

    - ASCII alphanumerics: `[a-zA-Z0-9]`
    - Special characters in the set `'!@#$%^&*()+[]{}<>/.,;:_-~`= \\||`

    In other words, MD5 keys may contain any printable ASCII character aside from
    newline (0x0A), quotation mark (`"`), vertical tab (0x0B), carriage return
    (0x0D), tab (0x09), form feed (0x0C), and the question mark (`?`). Requests
    specifying an MD5 key with one or more of these disallowed characters will be
    rejected.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal, TypeAlias

__all__ = ["TTL"]

TTL: TypeAlias = Union[float, Literal[1]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.addressing.prefixes import advertisement_status_edit_params
from ....types.addressing.prefixes.advertisement_status_get_response import AdvertisementStatusGetResponse
from ....types.addressing.prefixes.advertisement_status_edit_response import AdvertisementStatusEditResponse

__all__ = ["AdvertisementStatusResource", "AsyncAdvertisementStatusResource"]


class AdvertisementStatusResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AdvertisementStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AdvertisementStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdvertisementStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AdvertisementStatusResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def edit(
        self,
        prefix_id: str,
        *,
        account_id: str,
        advertised: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AdvertisementStatusEditResponse]:
        """
        Advertise or withdraw the BGP route for a prefix.

        **Deprecated:** Prefer the BGP Prefixes endpoints, which additionally allow for
        advertising and withdrawing subnets of an IP prefix.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          advertised: Advertisement status of the prefix. If `true`, the BGP route for the prefix is
              advertised to the Internet. If `false`, the BGP route is withdrawn.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/status",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=maybe_transform(
                {"advertised": advertised}, advertisement_status_edit_params.AdvertisementStatusEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AdvertisementStatusEditResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[AdvertisementStatusEditResponse]], ResultWrapper[AdvertisementStatusEditResponse]
            ),
        )

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AdvertisementStatusGetResponse]:
        """
        View the current advertisement state for a prefix.

        **Deprecated:** Prefer the BGP Prefixes endpoints, which additionally allow for
        advertising and withdrawing subnets of an IP prefix.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/status",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AdvertisementStatusGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AdvertisementStatusGetResponse]], ResultWrapper[AdvertisementStatusGetResponse]),
        )


class AsyncAdvertisementStatusResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAdvertisementStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAdvertisementStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdvertisementStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAdvertisementStatusResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def edit(
        self,
        prefix_id: str,
        *,
        account_id: str,
        advertised: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AdvertisementStatusEditResponse]:
        """
        Advertise or withdraw the BGP route for a prefix.

        **Deprecated:** Prefer the BGP Prefixes endpoints, which additionally allow for
        advertising and withdrawing subnets of an IP prefix.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          advertised: Advertisement status of the prefix. If `true`, the BGP route for the prefix is
              advertised to the Internet. If `false`, the BGP route is withdrawn.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/status",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=await async_maybe_transform(
                {"advertised": advertised}, advertisement_status_edit_params.AdvertisementStatusEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AdvertisementStatusEditResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[AdvertisementStatusEditResponse]], ResultWrapper[AdvertisementStatusEditResponse]
            ),
        )

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AdvertisementStatusGetResponse]:
        """
        View the current advertisement state for a prefix.

        **Deprecated:** Prefer the BGP Prefixes endpoints, which additionally allow for
        advertising and withdrawing subnets of an IP prefix.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/status",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AdvertisementStatusGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AdvertisementStatusGetResponse]], ResultWrapper[AdvertisementStatusGetResponse]),
        )


class AdvertisementStatusResourceWithRawResponse:
    def __init__(self, advertisement_status: AdvertisementStatusResource) -> None:
        self._advertisement_status = advertisement_status

        self.edit = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                advertisement_status.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                advertisement_status.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAdvertisementStatusResourceWithRawResponse:
    def __init__(self, advertisement_status: AsyncAdvertisementStatusResource) -> None:
        self._advertisement_status = advertisement_status

        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                advertisement_status.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                advertisement_status.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AdvertisementStatusResourceWithStreamingResponse:
    def __init__(self, advertisement_status: AdvertisementStatusResource) -> None:
        self._advertisement_status = advertisement_status

        self.edit = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                advertisement_status.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                advertisement_status.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAdvertisementStatusResourceWithStreamingResponse:
    def __init__(self, advertisement_status: AsyncAdvertisementStatusResource) -> None:
        self._advertisement_status = advertisement_status

        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                advertisement_status.edit,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                advertisement_status.get,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "IntegrationEditParams",
    "Config",
    "ConfigTeamsDevicesWorkspaceOneConfigRequest",
    "ConfigTeamsDevicesCrowdstrikeConfigRequest",
    "ConfigTeamsDevicesUptycsConfigRequest",
    "ConfigTeamsDevicesIntuneConfigRequest",
    "ConfigTeamsDevicesKolideConfigRequest",
    "ConfigTeamsDevicesTaniumConfigRequest",
    "ConfigTeamsDevicesSentineloneS2sConfigRequest",
    "ConfigTeamsDevicesCustomS2sConfigRequest",
]


class IntegrationEditParams(TypedDict, total=False):
    account_id: Required[str]

    config: Config
    """The configuration object containing third-party integration information."""

    interval: str
    """The interval between each posture check with the third-party API.

    Use `m` for minutes (e.g. `5m`) and `h` for hours (e.g. `12h`).
    """

    name: str
    """The name of the device posture integration."""

    type: Literal[
        "workspace_one", "crowdstrike_s2s", "uptycs", "intune", "kolide", "tanium_s2s", "sentinelone_s2s", "custom_s2s"
    ]
    """The type of device posture integration."""


class ConfigTeamsDevicesWorkspaceOneConfigRequest(TypedDict, total=False):
    api_url: Required[str]
    """The Workspace One API URL provided in the Workspace One Admin Dashboard."""

    auth_url: Required[str]
    """The Workspace One Authorization URL depending on your region."""

    client_id: Required[str]
    """The Workspace One client ID provided in the Workspace One Admin Dashboard."""

    client_secret: Required[str]
    """The Workspace One client secret provided in the Workspace One Admin Dashboard."""


class ConfigTeamsDevicesCrowdstrikeConfigRequest(TypedDict, total=False):
    api_url: Required[str]
    """The Crowdstrike API URL."""

    client_id: Required[str]
    """The Crowdstrike client ID."""

    client_secret: Required[str]
    """The Crowdstrike client secret."""

    customer_id: Required[str]
    """The Crowdstrike customer ID."""


class ConfigTeamsDevicesUptycsConfigRequest(TypedDict, total=False):
    api_url: Required[str]
    """The Uptycs API URL."""

    client_key: Required[str]
    """The Uptycs client secret."""

    client_secret: Required[str]
    """The Uptycs client secret."""

    customer_id: Required[str]
    """The Uptycs customer ID."""


class ConfigTeamsDevicesIntuneConfigRequest(TypedDict, total=False):
    client_id: Required[str]
    """The Intune client ID."""

    client_secret: Required[str]
    """The Intune client secret."""

    customer_id: Required[str]
    """The Intune customer ID."""


class ConfigTeamsDevicesKolideConfigRequest(TypedDict, total=False):
    client_id: Required[str]
    """The Kolide client ID."""

    client_secret: Required[str]
    """The Kolide client secret."""


class ConfigTeamsDevicesTaniumConfigRequest(TypedDict, total=False):
    api_url: Required[str]
    """The Tanium API URL."""

    client_secret: Required[str]
    """The Tanium client secret."""

    access_client_id: str
    """
    If present, this id will be passed in the `CF-Access-Client-ID` header when
    hitting the `api_url`.
    """

    access_client_secret: str
    """
    If present, this secret will be passed in the `CF-Access-Client-Secret` header
    when hitting the `api_url`.
    """


class ConfigTeamsDevicesSentineloneS2sConfigRequest(TypedDict, total=False):
    api_url: Required[str]
    """The SentinelOne S2S API URL."""

    client_secret: Required[str]
    """The SentinelOne S2S client secret."""


class ConfigTeamsDevicesCustomS2sConfigRequest(TypedDict, total=False):
    access_client_id: Required[str]
    """
    This id will be passed in the `CF-Access-Client-ID` header when hitting the
    `api_url`.
    """

    access_client_secret: Required[str]
    """
    This secret will be passed in the `CF-Access-Client-Secret` header when hitting
    the `api_url`.
    """

    api_url: Required[str]
    """The Custom Device Posture Integration API URL."""


Config: TypeAlias = Union[
    ConfigTeamsDevicesWorkspaceOneConfigRequest,
    ConfigTeamsDevicesCrowdstrikeConfigRequest,
    ConfigTeamsDevicesUptycsConfigRequest,
    ConfigTeamsDevicesIntuneConfigRequest,
    ConfigTeamsDevicesKolideConfigRequest,
    ConfigTeamsDevicesTaniumConfigRequest,
    ConfigTeamsDevicesSentineloneS2sConfigRequest,
    ConfigTeamsDevicesCustomS2sConfigRequest,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WANStaticAddressingParam"]


class WANStaticAddressingParam(TypedDict, total=False):
    """(optional) if omitted, use DHCP.

    Submit secondary_address when site is in high availability mode.
    """

    address: Required[str]
    """A valid CIDR notation representing an IP range."""

    gateway_address: Required[str]
    """A valid IPv4 address."""

    secondary_address: str
    """A valid CIDR notation representing an IP range."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["SlotListParams"]


class SlotListParams(TypedDict, total=False):
    account_id: Required[str]
    """Customer account tag"""

    address_contains: Optional[str]
    """If specified, only show slots with the given text in their address field"""

    cursor: Optional[int]

    limit: Optional[int]

    occupied: Optional[bool]
    """If specified, only show slots with a specific occupied/unoccupied state"""

    site: Optional[str]
    """If specified, only show slots located at the given site"""

    speed: Optional[str]
    """If specified, only show slots that support the given speed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .list_cursor import ListCursor as ListCursor
from .item_list_params import ItemListParams as ItemListParams
from .item_get_response import ItemGetResponse as ItemGetResponse
from .list_cursor_param import ListCursorParam as ListCursorParam
from .item_create_params import ItemCreateParams as ItemCreateParams
from .item_delete_params import ItemDeleteParams as ItemDeleteParams
from .item_list_response import ItemListResponse as ItemListResponse
from .item_update_params import ItemUpdateParams as ItemUpdateParams
from .item_create_response import ItemCreateResponse as ItemCreateResponse
from .item_delete_response import ItemDeleteResponse as ItemDeleteResponse
from .item_update_response import ItemUpdateResponse as ItemUpdateResponse
from .bulk_operation_get_response import BulkOperationGetResponse as BulkOperationGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .posture import (
    PostureResource,
    AsyncPostureResource,
    PostureResourceWithRawResponse,
    AsyncPostureResourceWithRawResponse,
    PostureResourceWithStreamingResponse,
    AsyncPostureResourceWithStreamingResponse,
)
from .integrations import (
    IntegrationsResource,
    AsyncIntegrationsResource,
    IntegrationsResourceWithRawResponse,
    AsyncIntegrationsResourceWithRawResponse,
    IntegrationsResourceWithStreamingResponse,
    AsyncIntegrationsResourceWithStreamingResponse,
)

__all__ = [
    "IntegrationsResource",
    "AsyncIntegrationsResource",
    "IntegrationsResourceWithRawResponse",
    "AsyncIntegrationsResourceWithRawResponse",
    "IntegrationsResourceWithStreamingResponse",
    "AsyncIntegrationsResourceWithStreamingResponse",
    "PostureResource",
    "AsyncPostureResource",
    "PostureResourceWithRawResponse",
    "AsyncPostureResourceWithRawResponse",
    "PostureResourceWithStreamingResponse",
    "AsyncPostureResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["LivestreamGetLivestreamSessionDetailsForSessionIDResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """The livestream ID."""

    created_at: Optional[datetime] = None
    """Timestamp the object was created at. The time is returned in ISO format."""

    err_message: Optional[str] = None
    """The server URL to which the RTMP encoder sends the video and audio data."""

    ingest_seconds: Optional[int] = None
    """Name of the livestream."""

    livestream_id: Optional[str] = None

    started_time: Optional[str] = None
    """Unique key for accessing each livestream."""

    stopped_time: Optional[str] = None
    """The web address that viewers can use to watch the livestream."""

    updated_at: Optional[str] = None
    """Timestamp the object was updated at. The time is returned in ISO format."""

    viewer_seconds: Optional[int] = None
    """Specifies if the livestream was disabled."""


class LivestreamGetLivestreamSessionDetailsForSessionIDResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "MemberCreateParams",
    "IAMCreateMemberWithRoles",
    "IAMCreateMemberWithPolicies",
    "IAMCreateMemberWithPoliciesPolicy",
    "IAMCreateMemberWithPoliciesPolicyPermissionGroup",
    "IAMCreateMemberWithPoliciesPolicyResourceGroup",
]


class IAMCreateMemberWithRoles(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    email: Required[str]
    """The contact email address of the user."""

    roles: Required[SequenceNotStr[str]]
    """Array of roles associated with this member."""

    status: Literal["accepted", "pending"]
    """Status of the member invitation.

    If not provided during creation, defaults to 'pending'. Changing from 'accepted'
    back to 'pending' will trigger a replacement of the member resource in
    Terraform.
    """


class IAMCreateMemberWithPolicies(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    email: Required[str]
    """The contact email address of the user."""

    policies: Required[Iterable[IAMCreateMemberWithPoliciesPolicy]]
    """Array of policies associated with this member."""

    status: Literal["accepted", "pending"]
    """Status of the member invitation.

    If not provided during creation, defaults to 'pending'. Changing from 'accepted'
    back to 'pending' will trigger a replacement of the member resource in
    Terraform.
    """


class IAMCreateMemberWithPoliciesPolicyPermissionGroup(TypedDict, total=False):
    """A group of permissions."""

    id: Required[str]
    """Identifier of the group."""


class IAMCreateMemberWithPoliciesPolicyResourceGroup(TypedDict, total=False):
    """A group of scoped resources."""

    id: Required[str]
    """Identifier of the group."""


class IAMCreateMemberWithPoliciesPolicy(TypedDict, total=False):
    access: Required[Literal["allow", "deny"]]
    """Allow or deny operations against the resources."""

    permission_groups: Required[Iterable[IAMCreateMemberWithPoliciesPolicyPermissionGroup]]
    """A set of permission groups that are specified to the policy."""

    resource_groups: Required[Iterable[IAMCreateMemberWithPoliciesPolicyResourceGroup]]
    """A list of resource groups that the policy applies to."""


MemberCreateParams: TypeAlias = Union[IAMCreateMemberWithRoles, IAMCreateMemberWithPolicies]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CatalogSyncCreateParams"]


class CatalogSyncCreateParams(TypedDict, total=False):
    account_id: Required[str]

    destination_type: Required[Literal["NONE", "ZERO_TRUST_LIST"]]

    name: Required[str]

    update_mode: Required[Literal["AUTO", "MANUAL"]]

    description: str

    policy: str

    forwarded: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from ..labeled_region import LabeledRegion
from ..lighthouse_report import LighthouseReport

__all__ = ["Test"]


class Test(BaseModel):
    __test__ = False
    id: Optional[str] = None
    """UUID."""

    date: Optional[datetime] = None

    desktop_report: Optional[LighthouseReport] = FieldInfo(alias="desktopReport", default=None)
    """The Lighthouse report."""

    mobile_report: Optional[LighthouseReport] = FieldInfo(alias="mobileReport", default=None)
    """The Lighthouse report."""

    region: Optional[LabeledRegion] = None
    """A test region with a label."""

    schedule_frequency: Optional[Literal["DAILY", "WEEKLY"]] = FieldInfo(alias="scheduleFrequency", default=None)
    """The frequency of the test."""

    url: Optional[str] = None
    """A URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .top_ases_params import TopAsesParams as TopAsesParams
from .top_ases_response import TopAsesResponse as TopAsesResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["AS112TimeseriesParams"]


class AS112TimeseriesParams(TypedDict, total=False):
    agg_interval: Annotated[Literal["15m", "1h", "1d", "1w"], PropertyInfo(alias="aggInterval")]
    """Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).

    Refer to
    [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).
    """

    continent: SequenceNotStr[str]
    """Filters results by continent.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    continents from results. For example, `-EU,NA` excludes results from EU, but
    includes results from NA.
    """

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[SequenceNotStr[str], PropertyInfo(alias="dateRange")]
    """Filters results by date range.

    For example, use `7d` and `7dcontrol` to compare this week with the previous
    week. Use this parameter or set specific start and end dates (`dateStart` and
    `dateEnd` parameters).
    """

    date_start: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    location: SequenceNotStr[str]
    """Filters results by location.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    locations from results. For example, `-US,PT` excludes results from the US, but
    includes results from PT.
    """

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""

    protocol: List[Literal["UDP", "TCP", "HTTPS", "TLS"]]
    """Filters results by DNS transport protocol."""

    query_type: Annotated[
        List[
            Optional[
                Literal[
                    "A",
                    "AAAA",
                    "A6",
                    "AFSDB",
                    "ANY",
                    "APL",
                    "ATMA",
                    "AXFR",
                    "CAA",
                    "CDNSKEY",
                    "CDS",
                    "CERT",
                    "CNAME",
                    "CSYNC",
                    "DHCID",
                    "DLV",
                    "DNAME",
                    "DNSKEY",
                    "DOA",
                    "DS",
                    "EID",
                    "EUI48",
                    "EUI64",
                    "GPOS",
                    "GID",
                    "HINFO",
                    "HIP",
                    "HTTPS",
                    "IPSECKEY",
                    "ISDN",
                    "IXFR",
                    "KEY",
                    "KX",
                    "L32",
                    "L64",
                    "LOC",
                    "LP",
                    "MAILA",
                    "MAILB",
                    "MB",
                    "MD",
                    "MF",
                    "MG",
                    "MINFO",
                    "MR",
                    "MX",
                    "NAPTR",
                    "NB",
                    "NBSTAT",
                    "NID",
                    "NIMLOC",
                    "NINFO",
                    "NS",
                    "NSAP",
                    "NSEC",
                    "NSEC3",
                    "NSEC3PARAM",
                    "NULL",
                    "NXT",
                    "OPENPGPKEY",
                    "OPT",
                    "PTR",
                    "PX",
                    "RKEY",
                    "RP",
                    "RRSIG",
                    "RT",
                    "SIG",
                    "SINK",
                    "SMIMEA",
                    "SOA",
                    "SPF",
                    "SRV",
                    "SSHFP",
                    "SVCB",
                    "TA",
                    "TALINK",
                    "TKEY",
                    "TLSA",
                    "TSIG",
                    "TXT",
                    "UINFO",
                    "UID",
                    "UNSPEC",
                    "URI",
                    "WKS",
                    "X25",
                    "ZONEMD",
                ]
            ]
        ],
        PropertyInfo(alias="queryType"),
    ]
    """Filters results by DNS query type."""

    response_code: Annotated[
        List[
            Literal[
                "NOERROR",
                "FORMERR",
                "SERVFAIL",
                "NXDOMAIN",
                "NOTIMP",
                "REFUSED",
                "YXDOMAIN",
                "YXRRSET",
                "NXRRSET",
                "NOTAUTH",
                "NOTZONE",
                "BADSIG",
                "BADKEY",
                "BADTIME",
                "BADMODE",
                "BADNAME",
                "BADALG",
                "BADTRUNC",
                "BADCOOKIE",
            ]
        ],
        PropertyInfo(alias="responseCode"),
    ]
    """Filters results by DNS response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.schema_validation import schema_get_params, schema_edit_params, schema_list_params, schema_create_params
from ...types.schema_validation.public_schema import PublicSchema
from ...types.schema_validation.schema_delete_response import SchemaDeleteResponse

__all__ = ["SchemasResource", "AsyncSchemasResource"]


class SchemasResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SchemasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SchemasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SchemasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SchemasResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        kind: Literal["openapi_v3"],
        name: str,
        source: str,
        validation_enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """Uploads a new OpenAPI schema for API Shield schema validation.

        The schema
        defines expected request/response formats for API endpoints.

        Args:
          zone_id: Identifier.

          kind: The kind of the schema

          name: A human-readable name for the schema

          source: The raw schema, e.g., the OpenAPI schema, either as JSON or YAML

          validation_enabled: An indicator if this schema is enabled

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/schema_validation/schemas", zone_id=zone_id),
            body=maybe_transform(
                {
                    "kind": kind,
                    "name": name,
                    "source": source,
                    "validation_enabled": validation_enabled,
                },
                schema_create_params.SchemaCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )

    def list(
        self,
        *,
        zone_id: str,
        omit_source: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        validation_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[PublicSchema]:
        """
        Lists all OpenAPI schemas uploaded to API Shield with pagination support.

        Args:
          zone_id: Identifier.

          omit_source: Omit the source-files of schemas and only retrieve their meta-data.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          validation_enabled: Filter for enabled schemas

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/schema_validation/schemas", zone_id=zone_id),
            page=SyncV4PagePaginationArray[PublicSchema],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "omit_source": omit_source,
                        "page": page,
                        "per_page": per_page,
                        "validation_enabled": validation_enabled,
                    },
                    schema_list_params.SchemaListParams,
                ),
            ),
            model=PublicSchema,
        )

    def delete(
        self,
        schema_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SchemaDeleteResponse:
        """Permanently removes an uploaded OpenAPI schema from API Shield.

        Operations using
        this schema will lose their validation rules.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SchemaDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SchemaDeleteResponse], ResultWrapper[SchemaDeleteResponse]),
        )

    def edit(
        self,
        schema_id: str,
        *,
        zone_id: str,
        validation_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """
        Modifies an existing OpenAPI schema in API Shield, updating the validation rules
        for associated API operations.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          validation_enabled: Flag whether schema is enabled for validation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return self._patch(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            body=maybe_transform({"validation_enabled": validation_enabled}, schema_edit_params.SchemaEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )

    def get(
        self,
        schema_id: str,
        *,
        zone_id: str,
        omit_source: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """
        Gets the contents and metadata of a specific OpenAPI schema uploaded to API
        Shield.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          omit_source: Omit the source-files of schemas and only retrieve their meta-data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"omit_source": omit_source}, schema_get_params.SchemaGetParams),
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )


class AsyncSchemasResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSchemasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSchemasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSchemasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSchemasResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        kind: Literal["openapi_v3"],
        name: str,
        source: str,
        validation_enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """Uploads a new OpenAPI schema for API Shield schema validation.

        The schema
        defines expected request/response formats for API endpoints.

        Args:
          zone_id: Identifier.

          kind: The kind of the schema

          name: A human-readable name for the schema

          source: The raw schema, e.g., the OpenAPI schema, either as JSON or YAML

          validation_enabled: An indicator if this schema is enabled

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/schema_validation/schemas", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "kind": kind,
                    "name": name,
                    "source": source,
                    "validation_enabled": validation_enabled,
                },
                schema_create_params.SchemaCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )

    def list(
        self,
        *,
        zone_id: str,
        omit_source: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        validation_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PublicSchema, AsyncV4PagePaginationArray[PublicSchema]]:
        """
        Lists all OpenAPI schemas uploaded to API Shield with pagination support.

        Args:
          zone_id: Identifier.

          omit_source: Omit the source-files of schemas and only retrieve their meta-data.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          validation_enabled: Filter for enabled schemas

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/schema_validation/schemas", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[PublicSchema],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "omit_source": omit_source,
                        "page": page,
                        "per_page": per_page,
                        "validation_enabled": validation_enabled,
                    },
                    schema_list_params.SchemaListParams,
                ),
            ),
            model=PublicSchema,
        )

    async def delete(
        self,
        schema_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SchemaDeleteResponse:
        """Permanently removes an uploaded OpenAPI schema from API Shield.

        Operations using
        this schema will lose their validation rules.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SchemaDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SchemaDeleteResponse], ResultWrapper[SchemaDeleteResponse]),
        )

    async def edit(
        self,
        schema_id: str,
        *,
        zone_id: str,
        validation_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """
        Modifies an existing OpenAPI schema in API Shield, updating the validation rules
        for associated API operations.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          validation_enabled: Flag whether schema is enabled for validation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return await self._patch(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            body=await async_maybe_transform(
                {"validation_enabled": validation_enabled}, schema_edit_params.SchemaEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )

    async def get(
        self,
        schema_id: str,
        *,
        zone_id: str,
        omit_source: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicSchema:
        """
        Gets the contents and metadata of a specific OpenAPI schema uploaded to API
        Shield.

        Args:
          zone_id: Identifier.

          schema_id: UUID.

          omit_source: Omit the source-files of schemas and only retrieve their meta-data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/schema_validation/schemas/{schema_id}", zone_id=zone_id, schema_id=schema_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"omit_source": omit_source}, schema_get_params.SchemaGetParams),
                post_parser=ResultWrapper[PublicSchema]._unwrapper,
            ),
            cast_to=cast(Type[PublicSchema], ResultWrapper[PublicSchema]),
        )


class SchemasResourceWithRawResponse:
    def __init__(self, schemas: SchemasResource) -> None:
        self._schemas = schemas

        self.create = to_raw_response_wrapper(
            schemas.create,
        )
        self.list = to_raw_response_wrapper(
            schemas.list,
        )
        self.delete = to_raw_response_wrapper(
            schemas.delete,
        )
        self.edit = to_raw_response_wrapper(
            schemas.edit,
        )
        self.get = to_raw_response_wrapper(
            schemas.get,
        )


class AsyncSchemasResourceWithRawResponse:
    def __init__(self, schemas: AsyncSchemasResource) -> None:
        self._schemas = schemas

        self.create = async_to_raw_response_wrapper(
            schemas.create,
        )
        self.list = async_to_raw_response_wrapper(
            schemas.list,
        )
        self.delete = async_to_raw_response_wrapper(
            schemas.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            schemas.edit,
        )
        self.get = async_to_raw_response_wrapper(
            schemas.get,
        )


class SchemasResourceWithStreamingResponse:
    def __init__(self, schemas: SchemasResource) -> None:
        self._schemas = schemas

        self.create = to_streamed_response_wrapper(
            schemas.create,
        )
        self.list = to_streamed_response_wrapper(
            schemas.list,
        )
        self.delete = to_streamed_response_wrapper(
            schemas.delete,
        )
        self.edit = to_streamed_response_wrapper(
            schemas.edit,
        )
        self.get = to_streamed_response_wrapper(
            schemas.get,
        )


class AsyncSchemasResourceWithStreamingResponse:
    def __init__(self, schemas: AsyncSchemasResource) -> None:
        self._schemas = schemas

        self.create = async_to_streamed_response_wrapper(
            schemas.create,
        )
        self.list = async_to_streamed_response_wrapper(
            schemas.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            schemas.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            schemas.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            schemas.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AuditListResponse", "Action", "Actor", "Organization", "Raw", "Resource"]


class Action(BaseModel):
    """Provides information about the action performed."""

    description: Optional[str] = None
    """A short description of the action performed."""

    result: Optional[str] = None
    """The result of the action, indicating success or failure."""

    time: Optional[datetime] = None
    """A timestamp indicating when the action was logged."""

    type: Optional[str] = None
    """A short string that describes the action that was performed."""


class Actor(BaseModel):
    """Provides details about the actor who performed the action."""

    id: Optional[str] = None
    """The ID of the actor who performed the action.

    If a user performed the action, this will be their User ID.
    """

    context: Optional[Literal["api_key", "api_token", "dash", "oauth", "origin_ca_key"]] = None

    email: Optional[str] = None
    """The email of the actor who performed the action."""

    ip_address: Optional[str] = None
    """The IP address of the request that performed the action."""

    token_id: Optional[str] = None
    """The API token ID when the actor context is an api_token or oauth."""

    token_name: Optional[str] = None
    """The API token name when the actor context is an api_token or oauth."""

    type: Optional[Literal["cloudflare_admin", "system", "user"]] = None
    """The type of actor."""


class Organization(BaseModel):
    """Contains organization related information."""

    id: Optional[str] = None
    """A unique identifier for the organization."""


class Raw(BaseModel):
    """Provides raw information about the request and response."""

    cf_rayid: Optional[str] = FieldInfo(alias="cf_ray_id", default=None)
    """The Cloudflare Ray ID for the request."""

    method: Optional[str] = None
    """The HTTP method of the request."""

    status_code: Optional[int] = None
    """The HTTP response status code returned by the API."""

    uri: Optional[str] = None
    """The URI of the request."""

    user_agent: Optional[str] = None
    """The client's user agent string sent with the request."""


class Resource(BaseModel):
    """Provides details about the affected resource."""

    id: Optional[str] = None
    """The unique identifier for the affected resource."""

    product: Optional[str] = None
    """The Cloudflare product associated with the resource."""

    request: Optional[object] = None

    response: Optional[object] = None

    scope: Optional[object] = None
    """The scope of the resource."""

    type: Optional[str] = None
    """The type of the resource."""


class AuditListResponse(BaseModel):
    id: Optional[str] = None
    """A unique identifier for the audit log entry."""

    action: Optional[Action] = None
    """Provides information about the action performed."""

    actor: Optional[Actor] = None
    """Provides details about the actor who performed the action."""

    organization: Optional[Organization] = None
    """Contains organization related information."""

    raw: Optional[Raw] = None
    """Provides raw information about the request and response."""

    resource: Optional[Resource] = None
    """Provides details about the affected resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ResourceExportParams"]


class ResourceExportParams(TypedDict, total=False):
    account_id: Required[str]

    desc: bool

    order_by: str
    """One of ["id", "resource_type", "region"]."""

    provider_id: str

    region: str

    resource_group: str

    resource_id: SequenceNotStr[str]

    resource_type: List[
        Literal[
            "aws_customer_gateway",
            "aws_egress_only_internet_gateway",
            "aws_internet_gateway",
            "aws_instance",
            "aws_network_interface",
            "aws_route",
            "aws_route_table",
            "aws_route_table_association",
            "aws_subnet",
            "aws_vpc",
            "aws_vpc_ipv4_cidr_block_association",
            "aws_vpn_connection",
            "aws_vpn_connection_route",
            "aws_vpn_gateway",
            "aws_security_group",
            "aws_vpc_security_group_ingress_rule",
            "aws_vpc_security_group_egress_rule",
            "aws_ec2_managed_prefix_list",
            "aws_ec2_transit_gateway",
            "aws_ec2_transit_gateway_prefix_list_reference",
            "aws_ec2_transit_gateway_vpc_attachment",
            "azurerm_application_security_group",
            "azurerm_lb",
            "azurerm_lb_backend_address_pool",
            "azurerm_lb_nat_pool",
            "azurerm_lb_nat_rule",
            "azurerm_lb_rule",
            "azurerm_local_network_gateway",
            "azurerm_network_interface",
            "azurerm_network_interface_application_security_group_association",
            "azurerm_network_interface_backend_address_pool_association",
            "azurerm_network_interface_security_group_association",
            "azurerm_network_security_group",
            "azurerm_public_ip",
            "azurerm_route",
            "azurerm_route_table",
            "azurerm_subnet",
            "azurerm_subnet_route_table_association",
            "azurerm_virtual_machine",
            "azurerm_virtual_network_gateway_connection",
            "azurerm_virtual_network",
            "azurerm_virtual_network_gateway",
            "google_compute_network",
            "google_compute_subnetwork",
            "google_compute_vpn_gateway",
            "google_compute_vpn_tunnel",
            "google_compute_route",
            "google_compute_address",
            "google_compute_global_address",
            "google_compute_router",
            "google_compute_interconnect_attachment",
            "google_compute_ha_vpn_gateway",
            "google_compute_forwarding_rule",
            "google_compute_network_firewall_policy",
            "google_compute_network_firewall_policy_rule",
            "cloudflare_static_route",
            "cloudflare_ipsec_tunnel",
        ]
    ]

    search: SequenceNotStr[str]

    v2: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ScanCreateParams"]


class ScanCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    url: Required[str]

    agent_readiness: Annotated[bool, PropertyInfo(alias="agentReadiness")]
    """Enable agent readiness checks."""

    country: Literal[
        "AF",
        "AL",
        "DZ",
        "AD",
        "AO",
        "AG",
        "AR",
        "AM",
        "AU",
        "AT",
        "AZ",
        "BH",
        "BD",
        "BB",
        "BY",
        "BE",
        "BZ",
        "BJ",
        "BM",
        "BT",
        "BO",
        "BA",
        "BW",
        "BR",
        "BN",
        "BG",
        "BF",
        "BI",
        "KH",
        "CM",
        "CA",
        "CV",
        "KY",
        "CF",
        "TD",
        "CL",
        "CN",
        "CO",
        "KM",
        "CG",
        "CR",
        "CI",
        "HR",
        "CU",
        "CY",
        "CZ",
        "CD",
        "DK",
        "DJ",
        "DM",
        "DO",
        "EC",
        "EG",
        "SV",
        "GQ",
        "ER",
        "EE",
        "SZ",
        "ET",
        "FJ",
        "FI",
        "FR",
        "GA",
        "GE",
        "DE",
        "GH",
        "GR",
        "GL",
        "GD",
        "GT",
        "GN",
        "GW",
        "GY",
        "HT",
        "HN",
        "HU",
        "IS",
        "IN",
        "ID",
        "IR",
        "IQ",
        "IE",
        "IL",
        "IT",
        "JM",
        "JP",
        "JO",
        "KZ",
        "KE",
        "KI",
        "KW",
        "KG",
        "LA",
        "LV",
        "LB",
        "LS",
        "LR",
        "LY",
        "LI",
        "LT",
        "LU",
        "MO",
        "MG",
        "MW",
        "MY",
        "MV",
        "ML",
        "MR",
        "MU",
        "MX",
        "FM",
        "MD",
        "MC",
        "MN",
        "MS",
        "MA",
        "MZ",
        "MM",
        "NA",
        "NR",
        "NP",
        "NL",
        "NZ",
        "NI",
        "NE",
        "NG",
        "KP",
        "MK",
        "NO",
        "OM",
        "PK",
        "PS",
        "PA",
        "PG",
        "PY",
        "PE",
        "PH",
        "PL",
        "PT",
        "QA",
        "RO",
        "RU",
        "RW",
        "SH",
        "KN",
        "LC",
        "VC",
        "WS",
        "SM",
        "ST",
        "SA",
        "SN",
        "RS",
        "SC",
        "SL",
        "SK",
        "SI",
        "SB",
        "SO",
        "ZA",
        "KR",
        "SS",
        "ES",
        "LK",
        "SD",
        "SR",
        "SE",
        "CH",
        "SY",
        "TW",
        "TJ",
        "TZ",
        "TH",
        "BS",
        "GM",
        "TL",
        "TG",
        "TO",
        "TT",
        "TN",
        "TR",
        "TM",
        "UG",
        "UA",
        "AE",
        "GB",
        "US",
        "UY",
        "UZ",
        "VU",
        "VE",
        "VN",
        "YE",
        "ZM",
        "ZW",
    ]
    """Country to geo egress from"""

    customagent: str

    custom_headers: Annotated[Dict[str, str], PropertyInfo(alias="customHeaders")]
    """Set custom headers."""

    referer: str

    screenshots_resolutions: Annotated[
        List[Literal["desktop", "mobile", "tablet"]], PropertyInfo(alias="screenshotsResolutions")
    ]
    """Take multiple screenshots targeting different device types."""

    visibility: Literal["Public", "Unlisted"]
    """
    The option `Public` means it will be included in listings like recent scans and
    search results. `Unlisted` means it will not be included in the aforementioned
    listings, users will need to have the scan's ID to access it. A a scan will be
    automatically marked as unlisted if it fails, if it contains potential PII or
    other sensitive material.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.brand_protection import query_bulk_params, query_create_params, query_delete_params

__all__ = ["QueriesResource", "AsyncQueriesResource"]


class QueriesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QueriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return QueriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return QueriesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        query_scan: bool | Omit = omit,
        query_tag: str | Omit = omit,
        max_time: Union[str, datetime, None] | Omit = omit,
        min_time: Union[str, datetime, None] | Omit = omit,
        body_scan: bool | Omit = omit,
        string_matches: object | Omit = omit,
        body_tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after creating new saved string queries

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template("/accounts/{account_id}/brand-protection/queries", account_id=account_id),
            body=maybe_transform(
                {
                    "max_time": max_time,
                    "min_time": min_time,
                    "body_scan": body_scan,
                    "string_matches": string_matches,
                    "body_tag": body_tag,
                },
                query_create_params.QueryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "query_scan": query_scan,
                        "query_tag": query_tag,
                    },
                    query_create_params.QueryCreateParams,
                ),
            ),
            cast_to=NoneType,
        )

    def delete(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        scan: bool | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after deleting saved string queries by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/accounts/{account_id}/brand-protection/queries", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "scan": scan,
                        "tag": tag,
                    },
                    query_delete_params.QueryDeleteParams,
                ),
            ),
            cast_to=NoneType,
        )

    def bulk(
        self,
        *,
        account_id: str,
        queries: Iterable[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after creating new saved string queries in bulk

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template("/accounts/{account_id}/brand-protection/queries/bulk", account_id=account_id),
            body=maybe_transform({"queries": queries}, query_bulk_params.QueryBulkParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncQueriesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQueriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncQueriesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        query_scan: bool | Omit = omit,
        query_tag: str | Omit = omit,
        max_time: Union[str, datetime, None] | Omit = omit,
        min_time: Union[str, datetime, None] | Omit = omit,
        body_scan: bool | Omit = omit,
        string_matches: object | Omit = omit,
        body_tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after creating new saved string queries

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template("/accounts/{account_id}/brand-protection/queries", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "max_time": max_time,
                    "min_time": min_time,
                    "body_scan": body_scan,
                    "string_matches": string_matches,
                    "body_tag": body_tag,
                },
                query_create_params.QueryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "query_scan": query_scan,
                        "query_tag": query_tag,
                    },
                    query_create_params.QueryCreateParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def delete(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        scan: bool | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after deleting saved string queries by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/accounts/{account_id}/brand-protection/queries", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "scan": scan,
                        "tag": tag,
                    },
                    query_delete_params.QueryDeleteParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def bulk(
        self,
        *,
        account_id: str,
        queries: Iterable[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Return a success message after creating new saved string queries in bulk

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template("/accounts/{account_id}/brand-protection/queries/bulk", account_id=account_id),
            body=await async_maybe_transform({"queries": queries}, query_bulk_params.QueryBulkParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class QueriesResourceWithRawResponse:
    def __init__(self, queries: QueriesResource) -> None:
        self._queries = queries

        self.create = to_raw_response_wrapper(
            queries.create,
        )
        self.delete = to_raw_response_wrapper(
            queries.delete,
        )
        self.bulk = to_raw_response_wrapper(
            queries.bulk,
        )


class AsyncQueriesResourceWithRawResponse:
    def __init__(self, queries: AsyncQueriesResource) -> None:
        self._queries = queries

        self.create = async_to_raw_response_wrapper(
            queries.create,
        )
        self.delete = async_to_raw_response_wrapper(
            queries.delete,
        )
        self.bulk = async_to_raw_response_wrapper(
            queries.bulk,
        )


class QueriesResourceWithStreamingResponse:
    def __init__(self, queries: QueriesResource) -> None:
        self._queries = queries

        self.create = to_streamed_response_wrapper(
            queries.create,
        )
        self.delete = to_streamed_response_wrapper(
            queries.delete,
        )
        self.bulk = to_streamed_response_wrapper(
            queries.bulk,
        )


class AsyncQueriesResourceWithStreamingResponse:
    def __init__(self, queries: AsyncQueriesResource) -> None:
        self._queries = queries

        self.create = async_to_streamed_response_wrapper(
            queries.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            queries.delete,
        )
        self.bulk = async_to_streamed_response_wrapper(
            queries.bulk,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .durable_object import DurableObject as DurableObject
from .object_list_params import ObjectListParams as ObjectListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .workers import (
    WorkersResource,
    AsyncWorkersResource,
    WorkersResourceWithRawResponse,
    AsyncWorkersResourceWithRawResponse,
    WorkersResourceWithStreamingResponse,
    AsyncWorkersResourceWithStreamingResponse,
)
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)

__all__ = [
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "WorkersResource",
    "AsyncWorkersResource",
    "WorkersResourceWithRawResponse",
    "AsyncWorkersResourceWithRawResponse",
    "WorkersResourceWithStreamingResponse",
    "AsyncWorkersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.intel import miscategorization_create_params
from ..._base_client import make_request_options
from ...types.intel.miscategorization_create_response import MiscategorizationCreateResponse

__all__ = ["MiscategorizationsResource", "AsyncMiscategorizationsResource"]


class MiscategorizationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MiscategorizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MiscategorizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MiscategorizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MiscategorizationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        content_adds: Iterable[int] | Omit = omit,
        content_removes: Iterable[int] | Omit = omit,
        indicator_type: Literal["domain", "ipv4", "ipv6", "url"] | Omit = omit,
        ip: Optional[str] | Omit = omit,
        security_adds: Iterable[int] | Omit = omit,
        security_removes: Iterable[int] | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MiscategorizationCreateResponse:
        """
        Allows you to submit requests to change a domain’s category.

        Requests that include category `169` (New Domains) or category `177` (Newly
        Seen) in any of `content_adds`, `content_removes`, `security_adds`, or
        `security_removes` will be rejected with a `400 Bad Request`. These categories
        are automatically managed and fall off 30 days after they are applied.

        Args:
          account_id: Identifier.

          content_adds: Content category IDs to add.

          content_removes: Content category IDs to remove.

          ip: Provide only if indicator_type is `ipv4` or `ipv6`.

          security_adds: Security category IDs to add.

          security_removes: Security category IDs to remove.

          url: Provide only if indicator_type is `domain` or `url`. Example if indicator_type
              is `domain`: `example.com`. Example if indicator_type is `url`:
              `https://example.com/news/`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/intel/miscategorization", account_id=account_id),
            body=maybe_transform(
                {
                    "content_adds": content_adds,
                    "content_removes": content_removes,
                    "indicator_type": indicator_type,
                    "ip": ip,
                    "security_adds": security_adds,
                    "security_removes": security_removes,
                    "url": url,
                },
                miscategorization_create_params.MiscategorizationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MiscategorizationCreateResponse,
        )


class AsyncMiscategorizationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMiscategorizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMiscategorizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMiscategorizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMiscategorizationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        content_adds: Iterable[int] | Omit = omit,
        content_removes: Iterable[int] | Omit = omit,
        indicator_type: Literal["domain", "ipv4", "ipv6", "url"] | Omit = omit,
        ip: Optional[str] | Omit = omit,
        security_adds: Iterable[int] | Omit = omit,
        security_removes: Iterable[int] | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MiscategorizationCreateResponse:
        """
        Allows you to submit requests to change a domain’s category.

        Requests that include category `169` (New Domains) or category `177` (Newly
        Seen) in any of `content_adds`, `content_removes`, `security_adds`, or
        `security_removes` will be rejected with a `400 Bad Request`. These categories
        are automatically managed and fall off 30 days after they are applied.

        Args:
          account_id: Identifier.

          content_adds: Content category IDs to add.

          content_removes: Content category IDs to remove.

          ip: Provide only if indicator_type is `ipv4` or `ipv6`.

          security_adds: Security category IDs to add.

          security_removes: Security category IDs to remove.

          url: Provide only if indicator_type is `domain` or `url`. Example if indicator_type
              is `domain`: `example.com`. Example if indicator_type is `url`:
              `https://example.com/news/`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/intel/miscategorization", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "content_adds": content_adds,
                    "content_removes": content_removes,
                    "indicator_type": indicator_type,
                    "ip": ip,
                    "security_adds": security_adds,
                    "security_removes": security_removes,
                    "url": url,
                },
                miscategorization_create_params.MiscategorizationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MiscategorizationCreateResponse,
        )


class MiscategorizationsResourceWithRawResponse:
    def __init__(self, miscategorizations: MiscategorizationsResource) -> None:
        self._miscategorizations = miscategorizations

        self.create = to_raw_response_wrapper(
            miscategorizations.create,
        )


class AsyncMiscategorizationsResourceWithRawResponse:
    def __init__(self, miscategorizations: AsyncMiscategorizationsResource) -> None:
        self._miscategorizations = miscategorizations

        self.create = async_to_raw_response_wrapper(
            miscategorizations.create,
        )


class MiscategorizationsResourceWithStreamingResponse:
    def __init__(self, miscategorizations: MiscategorizationsResource) -> None:
        self._miscategorizations = miscategorizations

        self.create = to_streamed_response_wrapper(
            miscategorizations.create,
        )


class AsyncMiscategorizationsResourceWithStreamingResponse:
    def __init__(self, miscategorizations: AsyncMiscategorizationsResource) -> None:
        self._miscategorizations = miscategorizations

        self.create = async_to_streamed_response_wrapper(
            miscategorizations.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["V2ListParams", "Meta"]


class V2ListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    continuation_token: Optional[str]
    """Continuation token to fetch next page.

    Passed as a query param when requesting List V2 api endpoint.
    """

    creator: Optional[str]
    """Internal user ID set within the creator field.

    Setting to empty string "" will return images where creator field is not set
    """

    meta: Meta

    per_page: float
    """Number of items per page"""

    sort_order: Literal["asc", "desc"]
    """Sorting order by upload time"""


class Meta(TypedDict, total=False):
    field_operator: Annotated[str, PropertyInfo(alias="<field>[<operator>]")]
    """Optional metadata filter(s). Multiple filters can be combined with AND logic.

    **Operators:**

    - `eq`, `eq:string`, `eq:number`, `eq:boolean` - Exact match
    - `in`, `in:string`, `in:number` - Match any value in pipe-separated list

    **Examples:**

    - `meta.status[eq]=active`
    - `meta.priority[eq:number]=5`
    - `meta.enabled[eq:boolean]=true`
    - `meta.region[in]=us-east|us-west|eu-west`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.hostnames.settings import tls_update_params
from ....types.hostnames.settings.setting import Setting
from ....types.hostnames.settings.tls_get_response import TLSGetResponse
from ....types.hostnames.settings.setting_value_param import SettingValueParam
from ....types.hostnames.settings.tls_delete_response import TLSDeleteResponse

__all__ = ["TLSResource", "AsyncTLSResource"]


class TLSResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TLSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TLSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TLSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TLSResourceWithStreamingResponse(self)

    def update(
        self,
        hostname: str,
        *,
        zone_id: str,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        value: SettingValueParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Setting]:
        """
        Update the tls setting value for the hostname.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          hostname: The hostname for which the tls settings are set.

          value: The TLS setting value. The type depends on the `setting_id` used in the request
              path:

              - `ciphers`: an array of allowed cipher suite strings in BoringSSL format (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: a string indicating the minimum TLS version — one of
                `"1.0"`, `"1.1"`, `"1.2"`, or `"1.3"` (e.g., `"1.2"`)
              - `http2`: a string indicating whether HTTP/2 is enabled — `"on"` or `"off"`
                (e.g., `"on"`)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/hostnames/settings/{setting_id}/{hostname}",
                zone_id=zone_id,
                setting_id=setting_id,
                hostname=hostname,
            ),
            body=maybe_transform({"value": value}, tls_update_params.TLSUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Setting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Setting]], ResultWrapper[Setting]),
        )

    def delete(
        self,
        hostname: str,
        *,
        zone_id: str,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TLSDeleteResponse]:
        """
        Delete the tls setting value for the hostname.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          hostname: The hostname for which the tls settings are set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/hostnames/settings/{setting_id}/{hostname}",
                zone_id=zone_id,
                setting_id=setting_id,
                hostname=hostname,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TLSDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TLSDeleteResponse]], ResultWrapper[TLSDeleteResponse]),
        )

    def get(
        self,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TLSGetResponse]:
        """
        List the requested TLS setting for the hostnames under this zone.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/hostnames/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
            page=SyncSinglePage[TLSGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TLSGetResponse,
        )


class AsyncTLSResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTLSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTLSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTLSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTLSResourceWithStreamingResponse(self)

    async def update(
        self,
        hostname: str,
        *,
        zone_id: str,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        value: SettingValueParam,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Setting]:
        """
        Update the tls setting value for the hostname.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          hostname: The hostname for which the tls settings are set.

          value: The TLS setting value. The type depends on the `setting_id` used in the request
              path:

              - `ciphers`: an array of allowed cipher suite strings in BoringSSL format (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: a string indicating the minimum TLS version — one of
                `"1.0"`, `"1.1"`, `"1.2"`, or `"1.3"` (e.g., `"1.2"`)
              - `http2`: a string indicating whether HTTP/2 is enabled — `"on"` or `"off"`
                (e.g., `"on"`)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/hostnames/settings/{setting_id}/{hostname}",
                zone_id=zone_id,
                setting_id=setting_id,
                hostname=hostname,
            ),
            body=await async_maybe_transform({"value": value}, tls_update_params.TLSUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Setting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Setting]], ResultWrapper[Setting]),
        )

    async def delete(
        self,
        hostname: str,
        *,
        zone_id: str,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TLSDeleteResponse]:
        """
        Delete the tls setting value for the hostname.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          hostname: The hostname for which the tls settings are set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/hostnames/settings/{setting_id}/{hostname}",
                zone_id=zone_id,
                setting_id=setting_id,
                hostname=hostname,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TLSDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TLSDeleteResponse]], ResultWrapper[TLSDeleteResponse]),
        )

    def get(
        self,
        setting_id: Literal["ciphers", "min_tls_version", "http2"],
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TLSGetResponse, AsyncSinglePage[TLSGetResponse]]:
        """
        List the requested TLS setting for the hostnames under this zone.

        Args:
          zone_id: Identifier.

          setting_id:
              The TLS Setting name. The value type depends on the setting:

              - `ciphers`: value is an array of cipher suite strings (e.g.,
                `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
              - `min_tls_version`: value is a TLS version string (`"1.0"`, `"1.1"`, `"1.2"`,
                or `"1.3"`)
              - `http2`: value is `"on"` or `"off"`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/hostnames/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
            page=AsyncSinglePage[TLSGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TLSGetResponse,
        )


class TLSResourceWithRawResponse:
    def __init__(self, tls: TLSResource) -> None:
        self._tls = tls

        self.update = to_raw_response_wrapper(
            tls.update,
        )
        self.delete = to_raw_response_wrapper(
            tls.delete,
        )
        self.get = to_raw_response_wrapper(
            tls.get,
        )


class AsyncTLSResourceWithRawResponse:
    def __init__(self, tls: AsyncTLSResource) -> None:
        self._tls = tls

        self.update = async_to_raw_response_wrapper(
            tls.update,
        )
        self.delete = async_to_raw_response_wrapper(
            tls.delete,
        )
        self.get = async_to_raw_response_wrapper(
            tls.get,
        )


class TLSResourceWithStreamingResponse:
    def __init__(self, tls: TLSResource) -> None:
        self._tls = tls

        self.update = to_streamed_response_wrapper(
            tls.update,
        )
        self.delete = to_streamed_response_wrapper(
            tls.delete,
        )
        self.get = to_streamed_response_wrapper(
            tls.get,
        )


class AsyncTLSResourceWithStreamingResponse:
    def __init__(self, tls: AsyncTLSResource) -> None:
        self._tls = tls

        self.update = async_to_streamed_response_wrapper(
            tls.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            tls.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            tls.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .resources.resources import (
    ResourcesResource,
    AsyncResourcesResource,
    ResourcesResourceWithRawResponse,
    AsyncResourcesResourceWithRawResponse,
    ResourcesResourceWithStreamingResponse,
    AsyncResourcesResourceWithStreamingResponse,
)
from .....types.api_gateway.labels import managed_get_params
from .....types.api_gateway.labels.managed_get_response import ManagedGetResponse

__all__ = ["ManagedResource", "AsyncManagedResource"]


class ManagedResource(SyncAPIResource):
    @cached_property
    def resources(self) -> ResourcesResource:
        return ResourcesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ManagedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ManagedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ManagedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ManagedResourceWithStreamingResponse(self)

    def get(
        self,
        name: str,
        *,
        zone_id: str,
        with_mapped_resource_counts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedGetResponse:
        """
        Retrieve managed label

        Args:
          zone_id: Identifier.

          name: The name of the label

          with_mapped_resource_counts: Include `mapped_resources` for each label

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not name:
            raise ValueError(f"Expected a non-empty value for `name` but received {name!r}")
        return self._get(
            path_template("/zones/{zone_id}/api_gateway/labels/managed/{name}", zone_id=zone_id, name=name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"with_mapped_resource_counts": with_mapped_resource_counts}, managed_get_params.ManagedGetParams
                ),
                post_parser=ResultWrapper[ManagedGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ManagedGetResponse], ResultWrapper[ManagedGetResponse]),
        )


class AsyncManagedResource(AsyncAPIResource):
    @cached_property
    def resources(self) -> AsyncResourcesResource:
        return AsyncResourcesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncManagedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncManagedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncManagedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncManagedResourceWithStreamingResponse(self)

    async def get(
        self,
        name: str,
        *,
        zone_id: str,
        with_mapped_resource_counts: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedGetResponse:
        """
        Retrieve managed label

        Args:
          zone_id: Identifier.

          name: The name of the label

          with_mapped_resource_counts: Include `mapped_resources` for each label

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not name:
            raise ValueError(f"Expected a non-empty value for `name` but received {name!r}")
        return await self._get(
            path_template("/zones/{zone_id}/api_gateway/labels/managed/{name}", zone_id=zone_id, name=name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"with_mapped_resource_counts": with_mapped_resource_counts}, managed_get_params.ManagedGetParams
                ),
                post_parser=ResultWrapper[ManagedGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ManagedGetResponse], ResultWrapper[ManagedGetResponse]),
        )


class ManagedResourceWithRawResponse:
    def __init__(self, managed: ManagedResource) -> None:
        self._managed = managed

        self.get = to_raw_response_wrapper(
            managed.get,
        )

    @cached_property
    def resources(self) -> ResourcesResourceWithRawResponse:
        return ResourcesResourceWithRawResponse(self._managed.resources)


class AsyncManagedResourceWithRawResponse:
    def __init__(self, managed: AsyncManagedResource) -> None:
        self._managed = managed

        self.get = async_to_raw_response_wrapper(
            managed.get,
        )

    @cached_property
    def resources(self) -> AsyncResourcesResourceWithRawResponse:
        return AsyncResourcesResourceWithRawResponse(self._managed.resources)


class ManagedResourceWithStreamingResponse:
    def __init__(self, managed: ManagedResource) -> None:
        self._managed = managed

        self.get = to_streamed_response_wrapper(
            managed.get,
        )

    @cached_property
    def resources(self) -> ResourcesResourceWithStreamingResponse:
        return ResourcesResourceWithStreamingResponse(self._managed.resources)


class AsyncManagedResourceWithStreamingResponse:
    def __init__(self, managed: AsyncManagedResource) -> None:
        self._managed = managed

        self.get = async_to_streamed_response_wrapper(
            managed.get,
        )

    @cached_property
    def resources(self) -> AsyncResourcesResourceWithStreamingResponse:
        return AsyncResourcesResourceWithStreamingResponse(self._managed.resources)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["OperationBulkCreateParams", "Body"]


class OperationBulkCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    endpoint: Required[str]
    """
    The endpoint which can contain path parameter templates in curly braces, each
    will be replaced from left to right with {varN}, starting with {var1}, during
    insertion. This will further be Cloudflare-normalized upon insertion. See:
    https://developers.cloudflare.com/rules/normalization/how-it-works/.
    """

    host: Required[str]
    """RFC3986-compliant host."""

    method: Required[Literal["GET", "POST", "HEAD", "OPTIONS", "PUT", "DELETE", "CONNECT", "PATCH", "TRACE"]]
    """The HTTP method used to access the endpoint."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ......_models import BaseModel

__all__ = ["EntryUpdateResponse"]


class EntryUpdateResponse(BaseModel):
    """Specify a content list entry to block."""

    id: Optional[str] = None
    """Specify the identifier of the hostname."""

    content: Optional[str] = None
    """Specify the CID or content path of content to block."""

    created_on: Optional[datetime] = None

    description: Optional[str] = None
    """Specify an optional description of the content list entry."""

    modified_on: Optional[datetime] = None

    type: Optional[Literal["cid", "content_path"]] = None
    """Specify the type of content list entry to block."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FilterCreateParams"]


class FilterCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    expression: Required[str]
    """The filter expression."""

    mode: Required[str]
    """The filter's mode. Must be one of 'enabled', 'disabled', 'monitoring'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PCAPFilterParam"]


class PCAPFilterParam(TypedDict, total=False):
    """The packet capture filter. When this field is empty, all packets are captured."""

    destination_address: str
    """The destination IP address of the packet."""

    destination_port: float
    """The destination port of the packet."""

    protocol: float
    """The protocol number of the packet."""

    source_address: str
    """The source IP address of the packet."""

    source_port: float
    """The source port of the packet."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .currents import (
    CurrentsResource,
    AsyncCurrentsResource,
    CurrentsResourceWithRawResponse,
    AsyncCurrentsResourceWithRawResponse,
    CurrentsResourceWithStreamingResponse,
    AsyncCurrentsResourceWithStreamingResponse,
)
from .aggregates import (
    AggregatesResource,
    AsyncAggregatesResource,
    AggregatesResourceWithRawResponse,
    AsyncAggregatesResourceWithRawResponse,
    AggregatesResourceWithStreamingResponse,
    AsyncAggregatesResourceWithStreamingResponse,
)

__all__ = [
    "CurrentsResource",
    "AsyncCurrentsResource",
    "CurrentsResourceWithRawResponse",
    "AsyncCurrentsResourceWithRawResponse",
    "CurrentsResourceWithStreamingResponse",
    "AsyncCurrentsResourceWithStreamingResponse",
    "AggregatesResource",
    "AsyncAggregatesResource",
    "AggregatesResourceWithRawResponse",
    "AsyncAggregatesResourceWithRawResponse",
    "AggregatesResourceWithStreamingResponse",
    "AsyncAggregatesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["SnapshotCreateResponse"]


class SnapshotCreateResponse(BaseModel):
    content: str
    """HTML content."""

    screenshot: str
    """Base64 encoded image."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["BillingUsageHistoryResponse", "History"]


class History(BaseModel):
    id: str

    aggregated_value: float

    end_time: float

    start_time: float


class BillingUsageHistoryResponse(BaseModel):
    history: List[History]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FilterUpdateParams"]


class FilterUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    description: str
    """An informative summary of the filter."""

    expression: str
    """The filter expression.

    For more information, refer to
    [Expressions](https://developers.cloudflare.com/ruleset-engine/rules-language/expressions/).
    """

    paused: bool
    """When true, indicates that the filter is currently paused."""

    ref: str
    """A short reference tag. Allows you to select related filters."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["BillingInvoiceHistoryResponse", "Invoice", "Pagination"]


class Invoice(BaseModel):
    amount_due: float

    amount_paid: float

    amount_remaining: float

    currency: str

    id: Optional[str] = None

    attempt_count: Optional[float] = None

    attempted: Optional[bool] = None

    auto_advance: Optional[bool] = None

    created: Optional[float] = None

    created_by: Optional[str] = None

    description: Optional[str] = None

    invoice_origin: Optional[str] = None

    invoice_pdf: Optional[str] = None

    status: Optional[str] = None


class Pagination(BaseModel):
    has_more: bool

    page: float

    per_page: float

    total_count: float


class BillingInvoiceHistoryResponse(BaseModel):
    invoices: List[Invoice]

    pagination: Pagination



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["StoreDeleteParams"]


class StoreDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """Account Identifier"""

    force: bool
    """
    When true, cascade-deletes all secrets in the store before deleting the store
    itself. Required when deleting a non-empty store. Without this parameter,
    attempting to delete a non-empty store returns 409.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .content_lists import (
    ContentListsResource,
    AsyncContentListsResource,
    ContentListsResourceWithRawResponse,
    AsyncContentListsResourceWithRawResponse,
    ContentListsResourceWithStreamingResponse,
    AsyncContentListsResourceWithStreamingResponse,
)
from .ipfs_universal_paths import (
    IPFSUniversalPathsResource,
    AsyncIPFSUniversalPathsResource,
    IPFSUniversalPathsResourceWithRawResponse,
    AsyncIPFSUniversalPathsResourceWithRawResponse,
    IPFSUniversalPathsResourceWithStreamingResponse,
    AsyncIPFSUniversalPathsResourceWithStreamingResponse,
)

__all__ = [
    "ContentListsResource",
    "AsyncContentListsResource",
    "ContentListsResourceWithRawResponse",
    "AsyncContentListsResourceWithRawResponse",
    "ContentListsResourceWithStreamingResponse",
    "AsyncContentListsResourceWithStreamingResponse",
    "IPFSUniversalPathsResource",
    "AsyncIPFSUniversalPathsResource",
    "IPFSUniversalPathsResourceWithRawResponse",
    "AsyncIPFSUniversalPathsResourceWithRawResponse",
    "IPFSUniversalPathsResourceWithStreamingResponse",
    "AsyncIPFSUniversalPathsResourceWithStreamingResponse",
]
